@@ -88,6 +88,8 @@ extern PGDLLEXPORT Datum ora_date_trunc(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum ora_date_round(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum ora_timestamptz_trunc(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum ora_timestamptz_round(PG_FUNCTION_ARGS);
+extern PGDLLEXPORT Datum ora_timestamp_trunc(PG_FUNCTION_ARGS);
+extern PGDLLEXPORT Datum ora_timestamp_round(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum orafce_sysdate(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum orafce_sessiontimezone(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum orafce_dbtimezone(PG_FUNCTION_ARGS);
@@ -180,6 +182,8 @@ extern PGDLLEXPORT Datum plvdate_set_nonbizday_day(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum plvdate_unset_nonbizday_day(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum plvdate_use_easter(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum plvdate_using_easter(PG_FUNCTION_ARGS);
+extern PGDLLEXPORT Datum plvdate_use_great_friday(PG_FUNCTION_ARGS);
+extern PGDLLEXPORT Datum plvdate_using_great_friday(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum plvdate_include_start(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum plvdate_including_start(PG_FUNCTION_ARGS);
 extern PGDLLEXPORT Datum plvdate_default_holidays(PG_FUNCTION_ARGS);
