@@ -103,6 +103,8 @@ PG_FUNCTION_INFO_V1(ora_date_trunc);
 PG_FUNCTION_INFO_V1(ora_date_round);
 PG_FUNCTION_INFO_V1(ora_timestamptz_trunc);
 PG_FUNCTION_INFO_V1(ora_timestamptz_round);
+PG_FUNCTION_INFO_V1(ora_timestamp_trunc);
+PG_FUNCTION_INFO_V1(ora_timestamp_round);
 
 /*
  * Search const value in char array
@@ -576,7 +578,7 @@ _ora_date_round(DateADT day, int f)
 Datum
 ora_to_date(PG_FUNCTION_ARGS)
 {
-	text *date_txt = PG_GETARG_TEXT_P(0);
+	text *date_txt = PG_GETARG_TEXT_PP(0);
 	Timestamp result;
 
 	if(nls_date_format && strlen(nls_date_format))
@@ -655,10 +657,20 @@ get_session_timezone(FunctionCallInfo fcinfo)
 		result = *((pg_tz **) extra);
 		fcinfo->flinfo->fn_extra = result;
 
-		free(extra);
+		/*
+		 * check_timezone allocates small block of pg_tz * size. This block
+		 * should be released by free(extra), but I cannot release memory
+		 * allocated by application in library on MS platform. So I have to
+		 * accept small memory leak - elsewhere exception - broken heap :(
+		 *
+		 *
+		 * cannot be called
+		free( extra );
+		 */
 	}
 
 	return result;
+
 #else
 
 	return session_timezone;
@@ -666,32 +678,18 @@ get_session_timezone(FunctionCallInfo fcinfo)
 #endif
 }
 
-Datum
-ora_timestamptz_trunc(PG_FUNCTION_ARGS)
+/*
+ * redotz is used only for timestamp with time zone
+ */
+static void
+tm_trunc(struct pg_tm *tm, text *fmt, bool *redotz)
 {
-	TimestampTz timestamp = PG_GETARG_TIMESTAMPTZ(0);
-	TimestampTz result;
-	text *fmt = PG_GETARG_TEXT_PP(1);
-	int tz;
-	fsec_t fsec;
-	struct pg_tm tt, *tm = &tt;
-	const char *tzn;
-	bool redotz = false;
 	int f;
 
-	if (TIMESTAMP_NOT_FINITE(timestamp))
-		PG_RETURN_TIMESTAMPTZ(timestamp);
-
 	f = ora_seq_search(VARDATA_ANY(fmt), date_fmt, VARSIZE_ANY_EXHDR(fmt));
 	CHECK_SEQ_SEARCH(f, "round/trunc format string");
 
-	if (timestamp2tm(timestamp, &tz, tm, &fsec, &tzn, NULL) != 0)
-	ereport(ERROR,
-			(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
-			 errmsg("timestamp out of range")));
-
 	tm->tm_sec = 0;
-	fsec = 0;
 
 	switch (f)
 	{
@@ -706,7 +704,7 @@ ora_timestamptz_trunc(PG_FUNCTION_ARGS)
 		&tm->tm_year, &tm->tm_mon, &tm->tm_mday);
 		tm->tm_hour = 0;
 		tm->tm_min = 0;
-		redotz = true;
+		*redotz = true;
 		break;
 	CASE_fmt_YYYY
 		tm->tm_mon = 1;
@@ -716,18 +714,42 @@ ora_timestamptz_trunc(PG_FUNCTION_ARGS)
 		tm->tm_mday = 1;
 	CASE_fmt_DDD
 		tm->tm_hour = 0;
-		redotz = true; /* for all cases >= DAY */
+		*redotz = true; /* for all cases >= DAY */
 	CASE_fmt_HH
 		tm->tm_min = 0;
 	}
+}
+
+Datum
+ora_timestamptz_trunc(PG_FUNCTION_ARGS)
+{
+	TimestampTz timestamp = PG_GETARG_TIMESTAMPTZ(0);
+	TimestampTz result;
+	text *fmt = PG_GETARG_TEXT_PP(1);
+	int tz;
+	fsec_t fsec;
+	struct pg_tm tt, *tm = &tt;
+	const char *tzn;
+	bool redotz = false;
+
+	if (TIMESTAMP_NOT_FINITE(timestamp))
+		PG_RETURN_TIMESTAMPTZ(timestamp);
+
+	if (timestamp2tm(timestamp, &tz, tm, &fsec, &tzn, NULL) != 0)
+		ereport(ERROR,
+					(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
+					 errmsg("timestamp out of range")));
+
+	tm_trunc(tm, fmt, &redotz);
+	fsec = 0;
 
 	if (redotz)
 		tz = DetermineTimeZoneOffset(tm, get_session_timezone(fcinfo));
 
-	if (tm2timestamp(tm, fsec, &tz, &result) != 0)
+	if (tm2timestamp(tm, fsec	, &tz, &result) != 0)
 		ereport(ERROR,
-				(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
-				 errmsg("timestamp out of range")));
+					(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
+					 errmsg("timestamp out of range")));
 
 	PG_RETURN_TIMESTAMPTZ(result);
 }
@@ -768,35 +790,15 @@ ora_date_round(PG_FUNCTION_ARGS)
 #define ROUND_MDAY(_tm_) \
 	do { if (rounded) _tm_->tm_mday += _tm_->tm_hour >= 12?1:0; } while(0)
 
-
-Datum
-ora_timestamptz_round(PG_FUNCTION_ARGS)
+static void
+tm_round(struct pg_tm *tm, text *fmt, bool *redotz)
 {
-	TimestampTz timestamp = PG_GETARG_TIMESTAMPTZ(0);
-	TimestampTz result;
-	text *fmt = PG_GETARG_TEXT_PP(1);
-	int tz;
-	fsec_t fsec;
-	struct pg_tm tt, *tm = &tt;
-	const char *tzn;
-	bool redotz = false;
-	bool rounded = true;
-	int f;
-
-	if (TIMESTAMP_NOT_FINITE(timestamp))
-		PG_RETURN_TIMESTAMPTZ(timestamp);
+	int 	f;
+	bool	rounded = true;
 
 	f = ora_seq_search(VARDATA_ANY(fmt), date_fmt, VARSIZE_ANY_EXHDR(fmt));
 	CHECK_SEQ_SEARCH(f, "round/trunc format string");
 
-	if (timestamp2tm(timestamp, &tz, tm, &fsec, &tzn, NULL) != 0)
-	ereport(ERROR,
-			(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
-			 errmsg("timestamp out of range")));
-
-	/* tm->tm_sec = 0; */
-	fsec = 0;
-
 	/* set rounding rule */
 	switch (f)
 	{
@@ -855,13 +857,13 @@ ora_timestamptz_round(PG_FUNCTION_ARGS)
 		&tm->tm_year, &tm->tm_mon, &tm->tm_mday);
 		tm->tm_hour = 0;
 		tm->tm_min = 0;
-		redotz = true;
+		*redotz = true;
 		break;
 	CASE_fmt_DDD
 		tm->tm_mday += (tm->tm_hour >= 12)?1:0;
 		tm->tm_hour = 0;
 		tm->tm_min = 0;
-		redotz = true;
+		*redotz = true;
 		break;
 	CASE_fmt_MI
 		tm->tm_min += (tm->tm_sec >= 30)?1:0;
@@ -873,6 +875,29 @@ ora_timestamptz_round(PG_FUNCTION_ARGS)
 	}
 
 	tm->tm_sec = 0;
+}
+
+Datum
+ora_timestamptz_round(PG_FUNCTION_ARGS)
+{
+	TimestampTz timestamp = PG_GETARG_TIMESTAMPTZ(0);
+	TimestampTz result;
+	text *fmt = PG_GETARG_TEXT_PP(1);
+	int tz;
+	fsec_t fsec;
+	struct pg_tm tt, *tm = &tt;
+	const char *tzn;
+	bool redotz = false;
+
+	if (TIMESTAMP_NOT_FINITE(timestamp))
+		PG_RETURN_TIMESTAMPTZ(timestamp);
+
+	if (timestamp2tm(timestamp, &tz, tm, &fsec, &tzn, NULL) != 0)
+		ereport(ERROR,
+					(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
+					 errmsg("timestamp out of range")));
+
+	tm_round(tm, fmt, &redotz);
 
 	if (redotz)
 		tz = DetermineTimeZoneOffset(tm, get_session_timezone(fcinfo));
@@ -885,6 +910,63 @@ ora_timestamptz_round(PG_FUNCTION_ARGS)
 	PG_RETURN_TIMESTAMPTZ(result);
 }
 
+Datum
+ora_timestamp_trunc(PG_FUNCTION_ARGS)
+{
+	Timestamp timestamp = PG_GETARG_TIMESTAMP(0);
+	Timestamp result;
+	text *fmt = PG_GETARG_TEXT_PP(1);
+	fsec_t fsec;
+	struct pg_tm tt, *tm = &tt;
+	bool redotz = false;
+
+	if (TIMESTAMP_NOT_FINITE(timestamp))
+		PG_RETURN_TIMESTAMP(timestamp);
+
+	if (timestamp2tm(timestamp, NULL, tm, &fsec, NULL, NULL) != 0)
+		ereport(ERROR,
+					(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
+					 errmsg("timestamp out of range")));
+
+	tm_trunc(tm, fmt, &redotz);
+	fsec = 0;
+
+	if (tm2timestamp(tm, fsec, NULL, &result) != 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
+				 errmsg("timestamp out of range")));
+
+	PG_RETURN_TIMESTAMP(result);
+}
+
+Datum
+ora_timestamp_round(PG_FUNCTION_ARGS)
+{
+	Timestamp timestamp = PG_GETARG_TIMESTAMP(0);
+	Timestamp result;
+	text *fmt = PG_GETARG_TEXT_PP(1);
+	fsec_t fsec;
+	struct pg_tm tt, *tm = &tt;
+	bool	redotz = false;
+
+	if (TIMESTAMP_NOT_FINITE(timestamp))
+		PG_RETURN_TIMESTAMPTZ(timestamp);
+
+	if (timestamp2tm(timestamp, NULL, tm, &fsec, NULL, NULL) != 0)
+		ereport(ERROR,
+					(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
+					 errmsg("timestamp out of range")));
+
+	tm_round(tm, fmt, &redotz);
+
+	if (tm2timestamp(tm, fsec, NULL, &result) != 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
+				 errmsg("timestamp out of range")));
+
+	PG_RETURN_TIMESTAMP(result);
+}
+
 /********************************************************************
  *
  * ora_sysdate - sysdate
