@@ -40,7 +40,6 @@
 #define RESULT_DATA	0
 #define RESULT_WAIT	1
 
-#define NOT_INITIALIZED -1
 #define ONE_YEAR (60*60*24*365)
 
 PG_FUNCTION_INFO_V1(dbms_pipe_pack_message_text);
@@ -123,7 +122,17 @@ typedef struct PipesFctx {
 
 typedef struct
 {
-	LWLockId shmem_lock;
+#if (PG_VERSION_NUM >= 90600)
+
+	int tranche_id;
+	LWLock shmem_lock;
+
+#else
+
+	LWLockId shmem_lockid;
+
+#endif
+
 	pipe *pipes;
 	alert_event *events;
 	alert_lock *locks;
@@ -140,12 +149,17 @@ message_buffer *input_buffer = NULL;
 pipe* pipes = NULL;
 
 #if PG_VERSION_NUM >= 90400
-LWLockId shmem_lock = NULL;
+
+#define NOT_INITIALIZED		NULL
+
 #else
-#define NOT_INITIALIZED -1
-LWLockId shmem_lock = NOT_INITIALIZED;
+
+#define NOT_INITIALIZED		-1
+
 #endif
 
+LWLockId shmem_lockid = NOT_INITIALIZED;;
+
 unsigned int sid;                                 /* session id */
 
 extern alert_event *events;
@@ -228,15 +242,38 @@ ora_lock_shmem(size_t size, int max_pipes, int max_events, int max_locks, bool r
 	{
 		sh_mem = ShmemInitStruct("dbms_pipe", size, &found);
 		if (sh_mem == NULL)
-			ereport(ERROR,
+			ereport(FATAL,
 					(errcode(ERRCODE_OUT_OF_MEMORY),
 					 errmsg("out of memory"),
 					 errdetail("Failed while allocation block %lu bytes in shared memory.", (unsigned long) size)));
 
 		if (!found)
 		{
-			shmem_lock = sh_mem->shmem_lock = LWLockAssign();
-			LWLockAcquire(sh_mem->shmem_lock, LW_EXCLUSIVE);
+
+#if PG_VERSION_NUM >= 90600
+
+			sh_mem->tranche_id = LWLockNewTrancheId();
+			LWLockInitialize(&sh_mem->shmem_lock, sh_mem->tranche_id);
+
+			{
+				static LWLockTranche tranche;
+
+				tranche.name = "orafce";
+				tranche.array_base = &sh_mem->shmem_lock;
+				tranche.array_stride = sizeof(LWLock);
+				LWLockRegisterTranche(sh_mem->tranche_id, &tranche);
+
+				shmem_lockid = &sh_mem->shmem_lock;
+			}
+
+#else
+
+			shmem_lockid = sh_mem->shmem_lockid = LWLockAssign();
+
+#endif
+
+			LWLockAcquire(shmem_lockid, LW_EXCLUSIVE);
+
 			sh_mem->size = size - sh_memory_size;
 			ora_sinit(sh_mem->data, size, true);
 			pipes = sh_mem->pipes = ora_salloc(max_pipes*sizeof(pipe));
@@ -261,11 +298,22 @@ ora_lock_shmem(size_t size, int max_pipes, int max_events, int max_locks, bool r
 			}
 
 		}
-		else if (sh_mem->shmem_lock != 0)
+		else if (pipes == NULL)
 		{
 			pipes = sh_mem->pipes;
-			shmem_lock = sh_mem->shmem_lock;
-			LWLockAcquire(sh_mem->shmem_lock, LW_EXCLUSIVE);
+
+#if PG_VERSION_NUM >= 90600
+
+			shmem_lockid = &sh_mem->shmem_lock;
+
+#else
+
+			shmem_lockid = sh_mem->shmem_lockid;
+
+#endif
+
+			LWLockAcquire(shmem_lockid, LW_EXCLUSIVE);
+
 			ora_sinit(sh_mem->data, sh_mem->size, reset);
 			sid = ++(sh_mem->sid);
 			events = sh_mem->events;
@@ -274,12 +322,8 @@ ora_lock_shmem(size_t size, int max_pipes, int max_events, int max_locks, bool r
 	}
 	else
 	{
-		LWLockAcquire(shmem_lock, LW_EXCLUSIVE);
+		LWLockAcquire(shmem_lockid, LW_EXCLUSIVE);
 	}
-/*
-	if (reset && pipes == NULL)
-		elog(ERROR, "Can't purge memory");
-*/
 
 	return pipes != NULL;
 }
@@ -306,7 +350,7 @@ find_pipe(text* pipe_name, bool* created, bool only_check)
 
 			if (pipes[i].creator != NULL && pipes[i].uid != GetUserId())
 			{
-				LWLockRelease(shmem_lock);
+				LWLockRelease(shmem_lockid);
 				ereport(ERROR,
 						(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
 						 errmsg("insufficient privilege"),
@@ -433,7 +477,7 @@ get_from_pipe(text *pipe_name, bool *found)
 		}
 	}
 
-	LWLockRelease(shmem_lock);
+	LWLockRelease(shmem_lockid);
 
 	return result;
 }
@@ -490,7 +534,7 @@ add_to_pipe(text *pipe_name, message_buffer *ptr, int limit, bool limit_is_valid
 		}
 		break;
 	}
-	LWLockRelease(shmem_lock);
+	LWLockRelease(shmem_lockid);
 	return result;
 }
 
@@ -911,7 +955,7 @@ dbms_pipe_unique_session_name (PG_FUNCTION_ARGS)
 
 		result = cstring_to_text_with_len(strbuf.data, strbuf.len);
 		pfree(strbuf.data);
-		LWLockRelease(shmem_lock);
+		LWLockRelease(shmem_lockid);
 
 		PG_RETURN_TEXT_P(result);
 	}
@@ -1022,7 +1066,7 @@ dbms_pipe_list_pipes(PG_FUNCTION_ARGS)
 		fctx->pipe_nth += 1;
 	}
 
-	LWLockRelease(shmem_lock);
+	LWLockRelease(shmem_lockid);
 	SRF_RETURN_DONE(funcctx);
 }
 
@@ -1071,7 +1115,7 @@ dbms_pipe_create_pipe (PG_FUNCTION_ARGS)
 		{
 			if (!created)
 			{
-				LWLockRelease(shmem_lock);
+				LWLockRelease(shmem_lockid);
 				ereport(ERROR,
 						(errcode(ERRCODE_DUPLICATE_OBJECT),
 						 errmsg("pipe creation error"),
@@ -1099,7 +1143,7 @@ dbms_pipe_create_pipe (PG_FUNCTION_ARGS)
 			p->limit = limit_is_valid ? limit : -1;
 			p->registered = true;
 
-			LWLockRelease(shmem_lock);
+			LWLockRelease(shmem_lockid);
 			PG_RETURN_VOID();
 		}
 	}
@@ -1152,7 +1196,7 @@ dbms_pipe_purge (PG_FUNCTION_ARGS)
 	{
 
 		remove_pipe(pipe_name, true);
-		LWLockRelease(shmem_lock);
+		LWLockRelease(shmem_lockid);
 
 		PG_RETURN_VOID();
 	}
@@ -1180,7 +1224,7 @@ dbms_pipe_remove_pipe (PG_FUNCTION_ARGS)
 	{
 
 		remove_pipe(pipe_name, false);
-		LWLockRelease(shmem_lock);
+		LWLockRelease(shmem_lockid);
 
 		PG_RETURN_VOID();
 	}
